/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module alu_op(op,
	      a,
	      b,
	      carry_in,
	      
	      alu_out,
	      zero_out,
	      dc_out,
	      carry_out);
   input [3:0] op;
   input [7:0] a, b;
   input       carry_in;
   
   output [7:0] alu_out;
   output 	zero_out, dc_out, carry_out;

   parameter ALU_ADD = 4'b0000;
   parameter ALU_SUB = 4'b0001;
   parameter ALU_AND = 4'b0010;
   parameter ALU_OR = 4'b0011;
   parameter ALU_XOR = 4'b0100;
   parameter ALU_COM = 4'b0101;
   parameter ALU_RR = 4'b0110;
   parameter ALU_RL = 4'b0111;
   parameter ALU_SWAP =4'b1000;
   parameter ALU_BC   =4'b1001;
   parameter ALU_BS   =4'b1010;
   parameter ALU_BTC = 4'b1011;
   parameter ALU_BTS = 4'b1100;
   
   reg [7:0] bitdecode;
   reg [8:0] out;
   
   always @(op or a or b or carry_in or bitdecode) begin
      case (op)
	ALU_ADD: out <= a+b;
	ALU_SUB: out <= a-b;
	ALU_AND: out <= {1'b0, a & b};
	ALU_OR: out <= {1'b0, (a | b)};
	ALU_XOR: out <=  {1'b0, (a ^ b)};
	ALU_COM: out <= {1'b0, ~a};
	ALU_RR: out <={a[0], carry_in, a[7:1]};
	ALU_RL: out <={a[7:0], carry_in};
	ALU_SWAP:out <= {1'b0, a[3:0], a[7:4]};
	ALU_BC: out <= {1'b0, a & (~bitdecode)};
	ALU_BS: out <= {1'b0, a | bitdecode};
	ALU_BTC:out <= {1'b0, a & bitdecode};
	ALU_BTS:out <= ((a & bitdecode) != 8'b0000_0000) ? 9'b0_0000_0000 : {carry_in, a};
	default : out <= {carry_in, a};
      endcase // case(op)
      
   
   end 
   assign alu_out = out[7:0];
   assign carry_out = (op == ALU_SUB)? ~out[8]:out[8];
   assign dc_out = 
	  (op == ALU_ADD) ? out[4]:
	  (op == ALU_SUB) ? out[4]:0;
   assign zero_out = (out[7:0] == 8'b0000_0000) ? 1:0;
   
   always @(b[7:5])
     case (b[7:5])
       0:bitdecode <= 8'b0000_0001;
       1:bitdecode <= 8'b0000_0010;
       2:bitdecode <= 8'b0000_0100;
       3:bitdecode <= 8'b0000_1000;
       4:bitdecode <= 8'b0001_0000;
       5:bitdecode <= 8'b0010_0000;
       6:bitdecode <= 8'b0100_0000;
       7:bitdecode <= 8'b1000_0000;
     endcase // case(b[7:5])
endmodule	      




