/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module alu_top(f,
	       k,
	       w,
	       carry_in,
	       
	       op_addwf,
	       op_andwf,
	       op_clrf,
	       op_clrw,
	       op_comf,
	       op_decf,
	       op_decfsz,
	       op_incf,
	       op_incfsz,
	       op_iorwf,
	       op_movf,
	       op_movwf,
	       op_rlf,
	       op_rrf,
	       op_subwf,
	       op_swapf,
	       op_xorwf,
	       
	       op_bcf,
	       op_bsf,
	       op_btfsc,
	       op_btfss,
	       
	       op_andlw,
	       op_iorlw,
	       op_movlw,
 	       op_retlw,
	       op_xorlw,


	       alu_out,
	       zero_out,
	       dc_out,
	       carry_out);
   input [7:0] f;
   input [7:0] k;
   input [7:0] w;
   input       carry_in;
   input       op_addwf,
	       op_andwf,
	       op_clrf,
	       op_clrw,
	       op_comf,
	       op_decf,
	       op_decfsz,
	       op_incf,
	       op_incfsz,
	       op_iorwf,
	       op_movf,
	       op_movwf,
	       op_rlf,
	       op_rrf,
	       op_subwf,
	       op_swapf,
	       op_xorwf,
	       
	       op_bcf,
	       op_bsf,
	       op_btfsc,
	       op_btfss,
	       
	       op_andlw,
	       op_iorlw,
	       op_movlw,
 	       op_retlw,
	       op_xorlw;
   output [7:0] alu_out;
   output	zero_out,
		dc_out,
		carry_out;

   wire [7:0]	alu_a, alu_b;
   wire [3:0] 	alu_op;

   mux_alu_a u_mux_alu_a(f, 
			 k, 
			 w, 
			 op_comf, 
			 op_decf,
			 op_decfsz,
			 op_incf,
			 op_incfsz,
			 op_movf,
			 op_rlf,
			 op_rrf,
			 op_subwf,
			 op_swapf,
			 op_bcf,
			 op_bsf,
			 op_btfsc,
			 op_btfss,
			 op_andlw,
			 op_iorlw,
			 op_movlw,
			 op_retlw,
			 op_xorlw,
			 
			 alu_a);
   mux_alu_b u_mux_alu_b(f,
			 k,
			 w,
			 op_clrf,
			 op_clrw,
			 op_decf,
			 op_decfsz,
			 op_incf,
			 op_incfsz,
			 op_movwf,
			 op_subwf,
			 op_bcf,
			 op_bsf,
			 op_btfsc,
			 op_btfss,
			 op_andlw,
			 op_iorlw,
			 op_movlw,
			 op_retlw,
			 op_xorlw,

			 alu_b
			 );

   alu_op_gen u_alu_op_gen(op_addwf,
			   op_andwf,
			   op_clrf,
			   op_clrw,
			   op_comf,
			   op_decf,
			   op_decfsz,
			   op_incf,
			   op_incfsz,
			   op_iorwf,
			   op_movf,
			   op_movwf,
			   op_rlf,
			   op_rrf,
			   op_subwf,
			   op_swapf,
			   op_xorwf,
	      
			   op_bcf,
			   op_bsf,
			   op_btfsc,
			   op_btfss,
	      
			   op_andlw,
			   op_iorlw,
			   op_movlw,
 			   op_retlw,
			   op_xorlw,
			   alu_op);
   
	
   alu_op u_alu_op(alu_op, 
		   alu_a, 
		   alu_b, 
		   carry_in, 
		   alu_out, 
		   zero_out, 
		   dc_out,
		   carry_out);
endmodule // alu_top

