/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module instruction_decoder(inst,

			   op_nop,
			   op_movwf,
			   op_clrw,
			   op_clrf,
			   op_subwf,    //  11       654   0
			   op_decf,     //  opcode    d  f

			   op_andwf,
			   op_xorwf,
			   op_addwf,
			   op_iorwf,
			   op_movf,
			   op_comf,
			   op_incf,
			   op_decfsz,
			   op_rrf,
			   op_rlf,
			   op_swapf,
			   op_incfsz,
			   
			   op_bcf,
			   op_bsf,
			   op_btfsc,
			   op_btfss,

			   op_option,
			   op_sleep,
			   op_clrwdt,

			   op_tris,
			   op_retlw,
			   op_call,
			   op_goto,
			   op_movlw,
			   op_iorlw,
			   op_andlw,
			   op_xorlw
			   );
   
   input [11:0] inst;
   output 	op_nop,
		op_movwf,
		op_clrw,
		op_clrf,
		op_subwf,    //  11       654   0
		op_decf,     //  opcode    d  f

		op_andwf,
		op_xorwf,
		op_addwf,
		op_iorwf,
		op_movf,
		op_comf,
		op_incf,
		op_decfsz,
		op_rrf,
		op_rlf,
		op_swapf,
		op_incfsz,
		
		op_bcf,
		op_bsf,
		op_btfsc,
		op_btfss,
		
		op_option,
		op_sleep,
		op_clrwdt,

		op_tris,
		op_retlw,
		op_call,
		op_goto,
		op_movlw,
		op_iorlw,
		op_andlw,
		op_xorlw;

   parameter 	NOP = 12'b0000_0000_0000;
   parameter 	MOVWF = 7'b0000_001;
   parameter 	CLRW = 12'b0000_0100_0000;
   parameter 	CLRF = 7'b0000_011;
   parameter 	SUBWF = 6'b0000_10;
   parameter 	DECF  = 6'b0000_11;
   parameter 	IORWF = 6'b0001_00;
   parameter 	ANDWF = 6'b0001_01;
   parameter 	XORWF = 6'b0001_10;
   parameter 	ADDWF = 6'b0001_11;
   parameter 	MOVF  = 6'b0010_00;
   parameter 	COMF  = 6'b0010_01;
   parameter 	INCF  = 6'b0010_10;
   parameter 	DECFSZ = 6'b0010_11;
   parameter 	RRF    = 6'b0011_00;
   parameter 	RLF    = 6'b0011_01;
   parameter 	SWAPF  = 6'b0011_10;
   parameter 	INCFSZ = 6'b0011_11;
   //bit-oriented file register operations
   parameter 	BCF    = 4'b0100;
   parameter 	BSF    = 4'b0101;
   parameter 	BTFSC  = 4'b0110;
   parameter 	BTFSS  = 4'b0111;
// literal and control operations
   parameter 	OPTION = 12'b0000_0000_0010;
   parameter 	SLEEP  = 12'b0000_0000_0011;
   parameter 	CLRWDT = 12'b0000_0000_0100;
   parameter 	TRIS   = 12'b0000_0000_0111;
   parameter 	RETLW  = 4'b1000;
   parameter 	CALL   = 4'b1001;
   parameter 	GOTO   = 3'b101;
   parameter 	MOVLW  = 4'b1100;
   parameter 	IORLW  = 4'b1101;
   parameter 	ANDLW  = 4'b1110;
   parameter 	XORLW  = 4'b1111;
   
   assign 	op_nop = (inst[11:0] == NOP) ? 1 : 0;
   assign 	op_movwf = (inst[11:5] == MOVWF) ? 1 : 0;
   assign 	op_clrw = (inst[11:0] == CLRW) ? 1:0;
   assign 	op_clrf = (inst[11:5] == CLRF) ? 1:0;
   assign 	op_subwf = (inst[11:6] == SUBWF) ? 1:0;
   assign 	op_decf = (inst[11:6] == DECF) ? 1:0;
   
   assign 	op_andwf = (inst[11:6] == ANDWF) ? 1:0;
   assign 	op_xorwf = (inst[11:6] == XORWF) ? 1:0;
   assign 	op_addwf = (inst[11:6] == ADDWF) ? 1:0;
   assign 	op_iorwf = (inst[11:6] == IORWF) ? 1:0;
   assign 	op_movf = (inst[11:6] == MOVF) ? 1:0;
   assign 	op_comf = (inst[11:6] == COMF) ? 1:0;
   assign 	op_incf = (inst[11:6] == INCF) ? 1:0;
   assign 	op_decfsz = (inst[11:6] == DECFSZ) ? 1:0;
   assign 	op_rrf = (inst[11:6] == RRF) ? 1:0;
   assign 	op_rlf = (inst[11:6] == RLF) ? 1:0;
   assign 	op_swapf = (inst[11:6] == SWAPF) ? 1:0;
   assign 	op_incfsz = (inst[11:6] == INCFSZ) ? 1:0;


   assign 	op_bcf = (inst[11:8] == BCF) ? 1:0;
   assign 	op_bsf = (inst[11:8] == BSF) ? 1:0;
   assign 	op_btfsc = (inst[11:8] == BTFSC) ? 1:0;
   assign 	op_btfss = (inst[11:8] == BTFSS) ? 1:0;

   assign 	op_option = (inst[11:0] == OPTION) ? 1:0;
   assign 	op_sleep = (inst[11:0] == SLEEP) ? 1:0;
   assign 	op_clrwdt = (inst[11:0] == CLRWDT) ? 1:0;


   assign 	op_tris = (inst[11:0] == TRIS) ? 1:0;
   assign 	op_retlw = (inst[11:8] == RETLW) ? 1:0;
   assign 	op_call = (inst[11:8] == CALL) ? 1:0;
   assign 	op_goto = (inst[11:9] == GOTO) ? 1:0;
   assign 	op_movlw = (inst[11:8] == MOVLW) ? 1:0;
   assign 	op_iorlw = (inst[11:8] == IORLW) ? 1:0;
   assign 	op_andlw = (inst[11:8] == ANDLW) ? 1:0;
   assign 	op_xorlw = (inst[11:8] == XORLW) ? 1:0;
   
endmodule // instruction_decoder
