/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module instruction_decoder_top(clk,
			       reset,
			       fsr_rs,
			       skip,
			       rom_data,

			       
			       op_nop,
			       op_movwf,
			       op_clrw,
			       op_clrf,
			       op_subwf,    //  11       654   0
			       op_decf,     //  opcode    d  f
			       
			       op_andwf,
			       op_xorwf,
			       op_addwf,
			       op_iorwf,
			       op_movf,
			       op_comf,
			       op_incf,
			       op_decfsz,
			       op_rrf,
			       op_rlf,
			       op_swapf,
			       op_incfsz,

			       op_bcf,
			       op_bsf,
			       op_btfsc,
			       op_btfss,

			       op_option,
			       op_sleep,
			       op_clrwdt,

			       op_tris,
			       op_retlw,
			       op_call,
			       op_goto,
			       op_movlw,
			       op_iorlw,
			       op_andlw,
			       op_xorlw,

			       k,
			       longk,
			       fsel,
			       d,
			       fwe,
			       c_write, 
			       z_write
			       );
   
   input clk, reset, skip;
   input [4:0] fsr_rs;
   
   input [11:0] rom_data;
   
   output op_nop,
	  op_movwf,
	  op_clrw,
	  op_clrf,
	  op_subwf,    //  11       654   0
	  op_decf,     //  opcode    d  f
			       
	  op_andwf,
	  op_xorwf,
	  op_addwf,
	  op_iorwf,
	  op_movf,
	  op_comf,
	  op_incf,
	  op_decfsz,
	  op_rrf,
	  op_rlf,
	  op_swapf,
	  op_incfsz,

	  op_bcf,
	  op_bsf,
	  op_btfsc,
	  op_btfss,

	  op_option,
	  op_sleep,
	  op_clrwdt,
	  
	  op_tris,
	  op_retlw,
	  op_call,
	  op_goto,
	  op_movlw,
	  op_iorlw,
	  op_andlw,
	  op_xorlw;
   output [7:0] k;
   output [8:0] longk;
   output [4:0] fsel;
   output 	d, fwe;
   output 	c_write, 
		z_write;

   wire [11:0] 	inst;
   
   
   instruction_reg u_ir(clk, reset, fsr_rs, op_call, op_goto, skip, rom_data, inst,k, longk, fsel, d);

   instruction_decoder u_id(inst, 		   
			    op_nop,
			    op_movwf,
			    op_clrw,
			    op_clrf,
			    op_subwf,
			    op_decf, 
			    
			    op_andwf,
			    op_xorwf,
			    op_addwf,
			    op_iorwf,
			    op_movf,
			    op_comf,
			    op_incf,
			    op_decfsz,
			    op_rrf,
			    op_rlf,
			    op_swapf,
			    op_incfsz,
			    
			    op_bcf,
			    op_bsf,
			    op_btfsc,
			    op_btfss,

			    op_option,
			    op_sleep,
			    op_clrwdt,
			    
			    op_tris,
			    op_retlw,
			    op_call,
			    op_goto,
			    op_movlw,
			    op_iorlw,
			    op_andlw,
			    op_xorlw);
   
   mux_fwe u_mux_fwe(d,
		     op_movwf, 
		     op_clrf,
		     op_subwf,
		     op_decf,
		     op_andwf,
		     op_xorwf,
		     op_addwf,
		     op_iorwf,
		     op_movf,
		     op_comf,
		     op_incf,
		     op_decfsz,
		     op_rrf,
		     op_rlf,
		     op_swapf,
		     op_incfsz,
		     op_bcf,
		     op_bsf,

		     fwe);

   mux_status_write u_mux_status_write(op_clrw,   //z
				       op_clrf,   //z
				       op_subwf,//c,dc,z
				       op_decf, //z
				       
				       op_andwf,//c,dc, z
				       op_xorwf,//z
				       op_addwf,//z
				       op_iorwf,//z
				       op_movf, //z
				       op_comf, //z
				       op_incf, //z
				       op_rrf,  //c
				       op_rlf,  //c

//				       op_sleep,///TO, /PD
//				       op_clrwdt,///TO, /PD
				       
				       op_iorlw,//z
				       op_andlw,//z
				       op_xorlw,//z
				       
				       c_write, 
				       z_write
				       );
endmodule
   
