/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module pic16c5x(clk_q1,
		clk_q2,
		reset,
		rom_data,  
		pa_in, 
		pb_in,  
		pc_in, 
		pa_out,
		pb_out, 
		pc_out, 
		rom_addr,

		w_out,
		f_out,
		status_out,
		fsr_out
		);
   input clk_q1, clk_q2;
   input reset;
   input [11:0] rom_data;
   input [7:0] 	pa_in, pb_in, pc_in;
   output [7:0] pa_out, pb_out, pc_out,
		w_out, f_out, status_out, fsr_out;
   output [10:0] rom_addr;

   //opcode of instruction
   wire 	 op_nop,
		 op_movwf,
		 op_clrw,
		 op_clrf,
		 op_subwf,
		 op_decf, 
		 
		 op_andwf,
		 op_xorwf,
		 op_addwf,
		 op_iorwf,
		 op_movf,
		 op_comf,
		 op_incf,
		 op_decfsz,
		 op_rrf,
		 op_rlf,
		 op_swapf,
		 op_incfsz,
		 
		 op_bcf,
		 op_bsf,
		 op_btfsc,
		 op_btfss,

		 op_option,
		 op_sleep,
		 op_clrwdt,

		 op_tris,
		 op_retlw,
		 op_call,
		 op_goto,
		 op_movlw,
		 op_iorlw,
		 op_andlw,
		 op_xorlw;

   wire 	 c_write, z_write;

   wire [7:0] 	 k;
   wire [8:0] 	 longk;
   wire [4:0] 	 fsel;
   wire 	 fwe;
 	 

   wire [7:0] 	 alu_out;
   wire 	 alu_z, alu_dc, alu_c;
	 
   program_counter u_pc(clk_q2,
			reset,

			alu_z,
			longk,
			status_out[6:5],
			k,
			fwe,
			fsel,
			f_out,
			
			op_call,
			op_goto,
			op_retlw,
			op_decfsz,
			op_incfsz,
			op_btfsc,
			op_btfss,

			rom_addr,
			skip);

   
   instruction_decoder_top u_id(clk_q1,
				reset,
				fsr_out[4:0],
				skip,
				rom_data,
			   
				op_nop,
				op_movwf,
				op_clrw,
				op_clrf,
				op_subwf,
				op_decf,
		 
				op_andwf,
				op_xorwf,
				op_addwf,
				op_iorwf,
				op_movf,
				op_comf,
				op_incf,
				op_decfsz,
				op_rrf,
				op_rlf,
				op_swapf,
				op_incfsz,
				
				op_bcf,
				op_bsf,
				op_btfsc,
				op_btfss,
				
				op_option,
				op_sleep,
				op_clrwdt,
				
				op_tris,
				op_retlw,
				op_call,
				op_goto,
				op_movlw,
				op_iorlw,
				op_andlw,
				op_xorlw,

				k,
				longk,
				fsel,
				d,
				fwe,
				c_write, 
				z_write
				);
   
   alu_top u_alu(f_out,
		 k,
		 w_out,
		 status_out[0],
	       
		 op_addwf,
		 op_andwf,
		 op_clrf,
		 op_clrw,
		 op_comf,
		 op_decf,
		 op_decfsz,
		 op_incf,
		 op_incfsz,
		 op_iorwf,
		 op_movf,
		 op_movwf,
		 op_rlf,
		 op_rrf,
		 op_subwf,
		 op_swapf,
		 op_xorwf,
		 
		 op_bcf,
		 op_bsf,
		 op_btfsc,
		 op_btfss,
		 
		 op_andlw,
		 op_iorlw,
		 op_movlw,
 		 op_retlw,
		 op_xorlw,
		 

		 alu_out,
		 alu_z,
		 alu_dc,
		 alu_c);
   
   reg_top u_reg(clk_q1,
		 clk_q2,
		 reset,
		 fsel,
		 fwe,
		 d,
	       
		 alu_out,
	       
		 pa_in,
		 pb_in,
		 pc_in,

		 alu_c,
		 alu_z,
		 alu_dc,

		 c_write,
		 z_write,
	       
		 op_addwf,
		 op_andwf,
		 op_clrf,
		 op_clrw,
		 op_comf,
		 op_decf,
		 op_decfsz,
		 op_incf,
		 op_incfsz,
		 op_iorwf,
		 op_movf,
		 op_movwf,
		 op_rlf,
		 op_rrf,
		 op_subwf,
		 op_swapf,
		 op_xorwf,
		 
		 op_bcf,
		 op_bsf,
		 
		 op_andlw,
		 op_iorlw,
		 op_movlw,
 		 op_retlw,
		 op_xorlw,
	       
	       
		 w_out,
		 f_out,
		 pa_out,
		 pb_out,
		 pc_out,
		 fsr_out,
		 status_out
		 );
endmodule // pic16c5x
