/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module reg_top(clk_q3,
	       clk_q4,
	       reset,
	       fsel,
	       fwe,
	       d,
	       
	       alu_out,
	       
	       portA_in,
	       portB_in,
	       portC_in,

	       alu_c,
	       alu_z,
	       alu_dc,

	       c_write,
	       z_write,
	       
	       op_addwf,
	       op_andwf,
	       op_clrf,
	       op_clrw,
	       op_comf,
	       op_decf,
	       op_decfsz,
	       op_incf,
	       op_incfsz,
	       op_iorwf,
	       op_movf,
	       op_movwf,
	       op_rlf,
	       op_rrf,
	       op_subwf,
	       op_swapf,
	       op_xorwf,
	       
	       op_bcf,
	       op_bsf,
//	       op_btfsc,
//	       op_btfss,
	       
	       op_andlw,
	       op_iorlw,
	       op_movlw,
 	       op_retlw,
	       op_xorlw,
	       
	       
	       wout,
	       fout,
	       portA_out,
	       portB_out,
	       portC_out,
	       fsr_out,
	       status_out
	       );
   input clk_q3, clk_q4,reset, fwe;
   input [4:0] fsel;
   input [7:0] alu_out,
	       portA_in,
	       portB_in,
	       portC_in;
   input       d,
	       alu_c,
	       alu_z,
	       alu_dc,

	       c_write,
	       z_write,
	       
	       op_addwf,
	       op_andwf,
	       op_clrf,
	       op_clrw,
	       op_comf,
	       op_decf,
	       op_decfsz,
	       op_incf,
	       op_incfsz,
	       op_iorwf,
	       op_movf,
	       op_movwf,
	       op_rlf,
	       op_rrf,
	       op_subwf,
	       op_swapf,
	       op_xorwf,
	       
	       op_bcf,
	       op_bsf,
//	       op_btfsc,
//	       op_btfss,
	       
	       op_andlw,
	       op_iorlw,
	       op_movlw,
 	       op_retlw,
	       op_xorlw;
   output [7:0] wout,
		fout,
		portA_out,
		portB_out,
		portC_out,
		fsr_out,
		status_out;
   

   wire [7:0]  mux_w_out, mux_f_out;
   wire [7:0]  tmr0_out, portA_out, portB_out, portC_out;
   

   reg_mux_w u_mux_w(alu_out, 
		     wout, 
		     d, 
		     op_clrw, 
		     op_retlw,
		     op_movlw,
		     op_iorlw,
		     op_andlw,
		     op_subwf,
		     op_decf,
		     op_iorwf,
		     op_andwf,
		     op_movf,
		     op_comf,
		     op_incf,
		     op_decfsz,
		     op_rrf,
		     op_rlf,
		     op_swapf,
		     op_incfsz,
		     op_xorwf,
		     op_addwf,
		     op_xorlw,
		     mux_w_out);
   
   reg_mux_f u_mux_f(alu_out,
		     fout,
		     d,
		     
		     op_clrf,
		     op_movwf,
		     op_bcf,
		     op_bsf,
		     op_subwf,
		     op_decf,
		     op_iorwf,
		     op_andwf,
		     op_xorwf,
		     op_addwf,
		     op_movf,
		     op_comf,
		     op_incf,
		     op_decfsz,
		     op_rrf,
		     op_rlf,
		     op_swapf,
		     op_incfsz,

		     mux_f_out);

   reg_w u_reg_w(clk_q4, 
		 reset, 
		 mux_w_out, 
		 wout);
   reg_general u_reg_general(clk_q3, 
			     clk_q4, 
			     fwe, 
			     fsel, 
			     portA_in, 
			     portB_in, 
			     portC_in,
			     fsr_out,
			     status_out,
			     mux_f_out,
			     fout);

   reg_status u_reg_status(clk_q4,
			   reset,
			   c_write,
			   z_write,

			   alu_c,
			   alu_dc,
			   alu_z,

			   fwe,
			   fsel,
			   mux_f_out,
			   status_out);

   reg_fsr u_reg_fsr(clk_q4,
		     reset,
		     fsel,
		     fwe,
		     mux_f_out,
		     fsr_out);
   reg_io u_reg_io(clk_q4,
		   reset,
		   fwe,
		   fsel,
		   mux_f_out,
		   tmr0_out,
		   portA_out,
		   portB_out,
		   portC_out);
endmodule // reg_top



